@@ -7,6 +7,7 @@ module Network.DNS.LookupRaw (
   -- * Lookups returning DNS Messages
   , lookupRaw
   , lookupRawCtl
+  , lookupRawCtlRecv
   -- * DNS Message procesing
   , fromDNSMessage
   ) where
@@ -14,6 +15,7 @@ module Network.DNS.LookupRaw (
 import Data.Hourglass (timeAdd, Seconds)
 import Prelude hiding (lookup)
 import Time.System (timeCurrent)
+import Network.Socket (Socket)
 
 import Network.DNS.IO
 import Network.DNS.Imports hiding (lookup)
@@ -150,7 +152,7 @@ cacheNegative cconf c key v ans = case soas of
     soas = filter (SOA `isTypeOf`) $ authority ans
 
 insertNegative :: CacheConf -> Cache -> Key -> Entry -> TTL -> IO ()
-insertNegative CacheConf{..} c k v ttl = when (ttl /= 0) $ do
+insertNegative _ c k v ttl = when (ttl /= 0) $ do
     ctime <- timeCurrent
     let tim = ctime `timeAdd` life
     insertCache k tim v c
@@ -253,7 +255,20 @@ lookupRawCtl :: Resolver      -- ^ Resolver obtained via 'withResolver'
              -> TYPE          -- ^ Query RRtype
              -> QueryControls -- ^ Query flag and EDNS overrides
              -> IO (Either DNSError DNSMessage)
-lookupRawCtl rslv dom typ ctls = resolve dom typ rslv ctls receive
+lookupRawCtl rslv dom typ ctls = resolve rslv dom typ ctls receive
+
+-- | Similar to 'lookupRawCtl', but the recv action can be replaced with
+-- something other than `Network.DNS.IO.receive`.
+-- For example, in an environment where frequent retrieval of the current time
+-- is a performance issue, you can pass the time from outside instead of
+-- having `Network.DNS.IO.receive` retrieve the current time.
+lookupRawCtlRecv :: Resolver                  -- ^ Resolver obtained via 'withResolver'
+                 -> Domain                    -- ^ Query domain
+                 -> TYPE                      -- ^ Query RRtype
+                 -> QueryControls             -- ^ Query flag and EDNS overrides
+                 -> (Socket -> IO DNSMessage) -- ^ Action to receive message from socket
+                 -> IO (Either DNSError DNSMessage)
+lookupRawCtlRecv = resolve
 
 ----------------------------------------------------------------
 
